__author__ = 'Merlen Gross'
__version__ = '1.1.1'
__email__ = 'merlengross@gmail.com'
__contact__ = 'https://vk.com/merlen_gross'
__license__ = 'Apache License, Version 2.0'

import re

import requests


class API(object):
    def __init__(self, login=None, password=None, token=None,
                 app=4666548, scope=1522847, version='5.27'):
        """Authorization and make request to vk.com API.

        :param login: (optional) The login on vk.com.
        :param password: (optional) The password on vk.com.
        :param token: (optional) The oauth-token on vk.com.
        :param app_id: (optional) The application id on vk.com.
        :param scope: (optional) The application scope.
        :param version: (optional) The version API.
        """

        self.login = login
        self.password = password
        self.token = token
        self.app = app
        self.scope = scope
        self.version = version

        self.session = requests.Session()
        self.session.headers = {
            'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) '
            'AppleWebKit/537.36 (KHTML, like Gecko) '
            'Chrome/38.0.2125.122 Safari/537.36'
        }

        if self.login and self.password:
            self.auth()

    def auth(self):
        """ Authorization on vk.com. """

        url = 'https://login.vk.com/'
        values = {
            'act': 'login',
            'utf8': '1',
            'email': self.login,
            'pass': self.password
        }

        response = self.session.post(url, data=values)

        if 'feed' in response.url:
            self.oauth()

        return False


    def oauth(self):
        """ OAuth on vk.com. """

        url = 'https://oauth.vk.com/authorize'
        values = {
            'client_id': self.app,
            'scope': self.scope,
            'redirect_uri': 'https://oauth.vk.com/blank.html',
            'display': 'mobile',
            'v': self.version,
            'response_type': 'token'
        }

        response = self.session.get(url, params=values)

        if 'access_token' not in response.url:
            url = re.search('action="(.*)"', response.text).group(1)
            response = self.session.post(url, data=values)

        if 'access_token' in response.url:
            token = re.search('token=(\w+)', response.url).group(1)
            self.token = token

        return False

    def __getattr__(self, path):
        try:
            return self.__dict__[path]
        except KeyError:
            return Method(path, self.token, self.version)


class Method(object):
    """Create and save chain to make request.

    Example:
        api.users.get(users_ids=1)
    """

    def __init__(self, path='', token='', version=''):
        self.path = path
        self.token = token
        self.version = version

    def __getattr__(self, path):
        try:
            return self.__dict__[path]
        except KeyError:
            return Method('.'.join((self.path, path)), self.token,
                          self.version)

    def __call__(self, **params):
        return self.request(self.path, params)

    def request(self, method, params):
        """ Send request to API.

        :param method: The name of API method.
        :param params: The method parameters.
        """

        url = 'https://api.vk.com/method/' + method
        data = {
            'access_token': self.token,
            'v': self.version
        }
        data.update(params)

        return requests.post(url, data=data).json()

